#ifndef dlh_h
#define dlh_h

#include "config.h"

enum ArgType {
    ARG_NONE		= 0,
    ARG_OPTIONAL	= 1,	/* optional arg enclosed in [ ] */
    ARG_REQUIRED	= 2	/* required arg enclosed in { } */
};

#define ARG_TYPE_BITS	2	/* # of bits to hold an argument type */

#define ARG_MAX_SPECS	(8 * sizeof(unsigned) / ARG_TYPE_BITS)

#define ARG_OPT(pos)	(ARG_OPTIONAL << (ARG_TYPE_BITS * (pos)))
#define ARG_REQ(pos)	(ARG_REQUIRED << (ARG_TYPE_BITS * (pos)))

/* common argument patterns: */

#define ARG_REQ1		(ARG_REQ(0))
#define ARG_REQ2		(ARG_REQ1 | ARG_REQ(1))
#define ARG_REQ3		(ARG_REQ2 | ARG_REQ(2))

#define ARG_OPT1		(ARG_OPT(0))

#define ARG_OPT1_REQ1		(ARG_OPT(0) | ARG_REQ(1))
#define ARG_REQ1_OPT1_REQ1	(ARG_REQ(0) | ARG_OPT(1) | ARG_REQ(2))
#define ARG_REQ1_OPT1_REQ2	(ARG_REQ1_OPT1_REQ1 | ARG_REQ(3))

/*
 * figure:	f
 * footnote:	F
 * index:	i
 * section:	s
 * table:	t
 */

struct cmd;
struct env;

struct call {
    union {
	struct env *	env;
	struct cmd *	cmd;
    } u;
    char **		arg_list;	/* list of required arguments */
};

struct cmd {
    const char *	name;
    unsigned		pat;		/* pattern for argument parsing */
    void		(*op)(struct call *);
    const void *	arg;
};

struct env {
    const char *	name;
    unsigned		pat;		/* pattern for argument parsing */
    void		(*op)(struct call * c, int prologue);
    const char *	pro;
    const char *	epi;
};

struct env_stack {
    struct env_stack *	next;
    struct env *	e;
    int			ncleanup;
    int			nalloced;
    struct cleanup {
	void		(*op)(const void * arg);
	const void *	arg;
    } *cleanup;
};

struct file_stack {
    struct file_stack *	next;
    FILE *		file;
};

/*
 * Specifies how to typeset a column (cell) in a table:
 */
struct table {
    int		num_cols;
    int		want_border;
    struct cellspec {
	char	spec;	/* @, p, l, c, or r */
	char *	arg;	/* text in @{text} */
    } col[128];
};

/*
 * Structure for cross-referencing:
 */
struct xref_def {
    char *		name;		/* xref name, e.g., sec:example */
    char *		value;		/* NULL or, e.g., "3.6.3" */
    int			file_num;	/* defn is in file docFILE_NUM.html */
    /* [f]igure, [t]able, [s]ection, or [b]ibliography: */
    char		type;
};

struct xref_use {
    struct xref_def *	def;		/* defining cross-reference */
    long		pos;		/* position for value */
    int			file_num;	/* file to which POS refers to */
};

struct xref_tab {
    int			ndefs;
    int			ndefs_alloced;
    struct xref_def *	def;

    int			nuses;
    int			nuses_alloced;
    struct xref_use *	use;
};

struct dict_entry {
    const char *	key;
    int			file_num;
    int			label;
};

struct dict_tab {
    int			len;
    int			alloced;
    struct dict_entry *	dict;
};

#endif /* dlh_h */
